package cn.kdan.pdf.tech.core.controller;

import cn.kdan.pdf.tech.core.aspect.PreCheckRole;
import cn.kdan.pdf.tech.core.constant.VppDeviceConstant;
import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.model.VppCompany;
import cn.kdan.pdf.tech.core.pojo.vo.VppDeviceVO;
import cn.kdan.pdf.tech.core.service.VppCompanyService;
import cn.kdan.pdf.tech.core.service.VppDeviceService;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.util.MapUtils;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageInfo;
import constant.CommonConstant;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;
import utils.CommonUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.Map;

/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/vppDevice")
public class VppDeviceController {

    @Resource
    private VppDeviceService vppDeviceService;
    @Resource
    private VppCompanyService vppCompanyService;

    /**
     * 分页查询用户管理的团队的设备
     *
     * @param page 页数
     * @param pageSize 页码
     * @param productId 产品id
     * @param status 状态
     * @param teamId 团队id
     * @return PageInfo
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/page")
    public ResultMap<PageInfo<VppDeviceVO>> page(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                 @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                 @RequestParam(required = false) Integer productId,
                                                 @RequestParam(required = false) Integer status,
                                                 @RequestParam(required = false) String teamId,
                                                 @RequestParam(required = false) String queryString
                                    ) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,vppDeviceService.page(page,pageSize,productId,status,teamId,queryString));
    }

    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/download")
    public void downloadFailedUsingJson(HttpServletResponse response,
                                        @RequestParam(required = false) Integer productId,
                                        @RequestParam(required = false) String teamId,
                                        @RequestParam(required = false) Integer status,
                                        @RequestParam(required = false) String queryString) throws IOException {
        // 这里注意 有同学反应使用swagger 会导致各种问题，请直接用浏览器或者用postman
        try {
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
            VppCompany company = vppCompanyService.getUserCompany();
            String fileName = URLEncoder.encode(company.getName()+ " " + CommonUtils.getDateTime("yyyyMMdd") +" Device", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            // 这里需要设置不关闭流
            EasyExcel.write(response.getOutputStream(), VppDeviceVO.class)
                    .autoCloseStream(Boolean.FALSE)
                    .sheet("模板")
                    .doWrite(() -> {
                        // 分页查询数据
                        return vppDeviceService.listWithInfo( productId, status, teamId, queryString);
                    });
        } catch (Exception e) {
            // 重置response
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            Map<String, String> map = MapUtils.newHashMap();
            map.put("status", "failure");
            map.put("message", "下载文件失败" + e.getMessage());
            response.getWriter().println(JSON.toJSONString(map));
        }
    }


    /**
     * 解绑序列码和设备
     *
     * @param deviceId 设备id
     * @return    ResultMap
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @PostMapping("/cancel")
    public ResultMap<Boolean> cancel(@RequestParam String deviceId) {
        vppDeviceService.cancel(deviceId);
        return new ResultMap<>(CommonConstant.SUCCESS, VppDeviceConstant.SUCCESS_MSG_Canceled,Boolean.TRUE);
    }

}
