package cn.kdan.pdf.tech.core.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.kdan.pdf.tech.core.constant.AuthConstant;
import cn.kdan.pdf.tech.core.constant.VppMemberConstant;
import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.params.VppMemberRegisterParam;
import cn.kdan.pdf.tech.core.params.VppMemberResetPwdParam;
import cn.kdan.pdf.tech.core.params.VppMemberUpdatePwdParam;
import cn.kdan.pdf.tech.core.pojo.vo.VppMemberVO;
import cn.kdan.pdf.tech.core.service.VppMemberService;
import constant.CommonConstant;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/vppMember")
public class VppMemberController {

    @Resource
    private VppMemberService vppMemberService;

    /**
     * 用户注册
     *
     * @param param 用户注册入参
     * @return 用户注册出参
     */
    @PostMapping("/create")
    public ResultMap<VppMember> create(@RequestBody VppMemberRegisterParam param) {
        return new ResultMap<>(CommonConstant.SUCCESS, AuthConstant.SUCCESS_MSG_Register, vppMemberService.create(param));
    }

    /**
     * 用于忘记密码时重置密码
     *
     * @param params params
     * @return Boolean
     */
    @PostMapping("resetPassword")
    public ResultMap<Boolean> resetPassword(@RequestBody VppMemberResetPwdParam params) {
        return new ResultMap<>(CommonConstant.SUCCESS, VppMemberConstant.PASSWORD_RESET_SUCCESS, vppMemberService.resetPassword(params));
    }


    /**
     * 用户修改密码
     *
     * @param params params
     * @return Boolean
     */
    @PostMapping("updatePassword")
    public ResultMap<Boolean> updatePassword(@RequestBody @Valid VppMemberUpdatePwdParam params) {
        return new ResultMap<>(CommonConstant.SUCCESS, VppMemberConstant.PASSWORD_UPDATE_SUCCESS, vppMemberService.updatePassword(params));
    }

    /**
     * 添加管理员时判断用户是否存在
     * @param account 邮箱
     * @return Boolean
     */
    @PostMapping("/checkExist")
    public ResultMap<Boolean> checkExist(@RequestParam String account) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,vppMemberService.checkExist(account));
    }

    /**
     * 获取当前用户信息
     *
     * @return VppMember
     */
    @GetMapping("/getMemberInfo")
    public ResultMap<VppMemberVO> getMemberInfo() {
        VppMember vppMember = vppMemberService.getCurrentUser();
        VppMemberVO vppMemberVO = new VppMemberVO();
        VppMember byId = vppMemberService.getById(vppMember.getId());
        BeanUtil.copyProperties(byId, vppMemberVO);
        vppMemberVO.setDigestPassword(null);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppMemberVO);
    }
}
