package cn.kdan.pdf.tech.core.controller;

import cn.kdan.pdf.tech.core.aspect.PreCheckRole;
import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.model.VppCompany;
import cn.kdan.pdf.tech.core.pojo.vo.VppDeviceVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamAdminVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamMemberVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamMenberVO;
import cn.kdan.pdf.tech.core.service.VppCompanyService;
import cn.kdan.pdf.tech.core.service.VppRTeamMemberRoleService;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.util.MapUtils;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageInfo;
import constant.CommonConstant;
import org.springframework.web.bind.annotation.*;
import pojo.ResultMap;
import utils.CommonUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

/**
 * @author tangxiangan
 */
@RestController
@RequestMapping("/vppRTeamMemberRole")
public class VppRTeamMemberRoleController {

    @Resource
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;
    @Resource
    private VppCompanyService vppCompanyService;

    /**
     * 超管分页获取所在公司团队的管理员信息
     *
     * @param page     页码
     * @param pageSize 最大条数
     * @param teamId 团队id
     * @param role 角色 0超管 1团队管理员
     * @param queryString 名字或者邮箱
     * @return ResultMap
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/pageForAdmin")
    public ResultMap<PageInfo<VppTeamAdminVO>> pageForAdmin(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                            @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                            @RequestParam(required = false) String teamId,
                                                            @RequestParam(required = false) String role,
                                                            @RequestParam(required = false) String queryString) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppRTeamMemberRoleService.pageForAdmin(page, pageSize, teamId, role, queryString));
    }


    @GetMapping("/pageForMember")
    public ResultMap<PageInfo<VppTeamMemberVO>> pageForMember(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                              @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                              @RequestParam(required = false) String teamId,
                                                              @RequestParam(required = false) String status,
                                                              @RequestParam(required = false) String productCode,
                                                              @RequestParam(required = false) String queryString) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppRTeamMemberRoleService.pageForMember(page, pageSize, teamId, status, productCode,queryString));
    }


    @GetMapping("/download")
    public void downloadFailedUsingJson(HttpServletResponse response,
                                        @RequestParam(required = false) String productCode,
                                        @RequestParam(required = false) String teamId,
                                        @RequestParam(required = false) String status,
                                        @RequestParam(required = false) String queryString) throws IOException {
        // 这里注意 有同学反应使用swagger 会导致各种问题，请直接用浏览器或者用postman
        try {
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
            VppCompany company = vppCompanyService.getUserCompany();
            String fileName = URLEncoder.encode(company.getName()+ " " + CommonUtils.getDateTime("yyyyMMdd") +" Member", "UTF-8").replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            // 这里需要设置不关闭流
            EasyExcel.write(response.getOutputStream(), VppTeamMemberVO.class)
                    .autoCloseStream(Boolean.FALSE)
                    .sheet("模板")
                    .doWrite(() -> {
                        // 分页查询数据
                        return vppRTeamMemberRoleService.listForMember( teamId, status, productCode, queryString);
                    });
        } catch (Exception e) {
            // 重置response
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            Map<String, String> map = MapUtils.newHashMap();
            map.put("status", "failure");
            map.put("message", "下载文件失败" + e.getMessage());
            response.getWriter().println(JSON.toJSONString(map));
        }
    }


    /**
     * 获取团队下所有用户，根据邮箱去重
     *
     * @param teamId 团队id
     * @return List
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/listByTeamId")
    public ResultMap<List<VppTeamAdminVO>> listByTeamId(@RequestParam String teamId) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppRTeamMemberRoleService.listByTeamId(teamId));
    }

    /**
     * 获取当前登录用户的公司所有用户，除超管
     * @return List
     */
    @PreCheckRole(roles = {RoleEnum.TEAM_ADMIN,RoleEnum.SUPER_ADMIN})
    @GetMapping("/getComponyMenber")
    public ResultMap<List<VppTeamMenberVO>> getComponyMenber() {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppCompanyService.getComponyMenber());
    }

    /**
     * 修改团队成员时查询用户所在团队id
     * @param memberId
     * @return
     */
    @GetMapping("/getMemberTeam")
    public ResultMap<String> getMemberTeam(@RequestParam String memberId) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, vppRTeamMemberRoleService.getMemberTeam(memberId));
    }

    @PostMapping("/logOutUserNoAdmin")
    public ResultMap<Boolean> logOutUserNoAdmin(@RequestParam String memberId) {
        vppRTeamMemberRoleService.logOutUserNoAdmin(memberId);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS, Boolean.TRUE);
    }

}
