package cn.kdan.pdf.tech.core.service;

import cn.kdan.pdf.tech.core.model.VppTeam;
import cn.kdan.pdf.tech.core.pojo.dto.CreateTeamDTO;
import cn.kdan.pdf.tech.core.pojo.dto.EditTeamDTO;
import cn.kdan.pdf.tech.core.pojo.vo.TeamInfoVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamListVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamVO;
import com.github.pagehelper.PageInfo;
import org.springframework.web.bind.annotation.RequestParam;
import pojo.ResultMap;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

/**
 * @author tangxiangan
 */
public interface VppTeamService {

    /**
     * 团队管理列表-分页
     * @param teamId 团队id
     * @param keyword 模糊查询关键字
     * @return 团队数据
     */
    PageInfo<VppTeamVO> getManageTeamList(Integer page, Integer pageSize, String teamId, String keyword);

    /**
     * 导出团队管理列表-不分页
     * @param teamId 团队id
     * @param keyword 模糊查询关键字
     * @return
     */
    List<VppTeamVO> exportManageTeamList(String teamId, String keyword);

    void createTeam(CreateTeamDTO dto);

    void editTeam(EditTeamDTO dto);

    TeamInfoVO getTeamProductInfo(String teamId);

    void deleteTeam(String teamId);

    /**
     * 添加团队管理员
     *
     * @param teamIds 团队id
     * @param account 用户邮箱
     */
    void sendEmailForAddTeamAdmin(List<String> teamIds, String account);


    /**
     * 编辑管理员把他加入其他团队
     *
     * @param memberId 用户id
     * @param teamIds  团队id
     */
    void editTeamAdmin(String memberId, List<String> teamIds);


    /**
     * 获取登录用户的所有团队
     *
     * @return List
     */
    List<String> getMemberTeamIds();

    /**
     * 查询用户管理的所有团队（用于团队成员编辑选择团队）
     *
     * @return List
     */
    List<VppTeam> listWithAdmin();

    /**
     * 加入团队（用于管理员或者普通用户收到邮件加入团队）
     *
     * @param memberId 用户id
     * @param teamIds  团队id
     * @param action   动作 CaptchaActionEnum
     * @param code     验证
     */
    void enterTeam(String memberId, List<String> teamIds, String action, String code);

    /**
     * 获取超管所在公司下所有的团队信息
     *
     * @param companyId 公司id
     * @return List
     */
    List<VppTeam> listByCompany(String companyId);

    /**
     * 查出用户管理的团队和公司下的所有团队
     *
     * @param memberId 用户id
     * @return VppTeamListVO
     */
    VppTeamListVO listForAdminEdit(String memberId);

    /**
     * 更新团队信息
     *
     * @param vppTeam 团队
     */
    void updateTeam(VppTeam vppTeam);

    /**
     * 根据id获取团队
     *
     * @param teamId 团队id
     * @return VppTeam
     */
    VppTeam getById(String teamId);

    List<VppTeam> getByTeamParam(String companyId, String teamId, List<String> licenseTeamIdList, List<String> manageTeamIdList);

    /**
     * 根据名称获取团队
     *
     * @param teamName 团队名
     * @return VppTeam
     */
    VppTeam getByCompanyAndName(String companyId, String teamName);


    /**
     * 获取默认团队
     *
     * @param memberId 用户id
     * @return VppTeam
     */
    VppTeam getDefaultTeam(String memberId);
}
