package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.enums.LicenseCodeStatusEnum;
import cn.kdan.pdf.tech.core.mapper.ext.ExtVppSubscriptionMapper;
import cn.kdan.pdf.tech.core.model.LicenseCodes;
import cn.kdan.pdf.tech.core.model.Subscriptions;
import cn.kdan.pdf.tech.core.model.SubscriptionsExample;
import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.pojo.vo.SubscriptionResellerVO;
import cn.kdan.pdf.tech.core.pojo.vo.SubscriptionVO;
import cn.kdan.pdf.tech.core.service.VppLicenseCodeService;
import cn.kdan.pdf.tech.core.service.VppMemberService;
import cn.kdan.pdf.tech.core.service.SubscriptionService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * 订阅
 *
 * @author tangxiangan
 */
@Slf4j
@Service
public class SubscriptionServiceImpl implements SubscriptionService {

    @Autowired
    private ExtVppSubscriptionMapper vppSubscriptionMapper;
    @Resource
    private VppMemberService vppMemberService;
    @Resource
    private VppLicenseCodeService licenseCodeService;


    @Override
    public PageInfo<SubscriptionVO> pageByCompany(Integer page, Integer pageSize, String code) {
        VppMember member = vppMemberService.getCurrentUser();
        PageHelper.startPage(page, pageSize);
        List<SubscriptionVO> subscriptionVOS = vppSubscriptionMapper.listByCompany(code, member.getCompanyId());
        List<String> subscriptionIdList = subscriptionVOS.stream().map(SubscriptionVO::getId).collect(Collectors.toList());
        List<LicenseCodes> bySubscriptionIds = licenseCodeService.getBySubscriptionIds(subscriptionIdList);
        // 获取剩余可激活设备数
        for (SubscriptionVO subscriptionVO : subscriptionVOS) {
            List<LicenseCodes> collect = bySubscriptionIds.stream().filter(c -> c.getSubscriptionId().equals(subscriptionVO.getId())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(collect)) {
                log.warn("当前订单未找到序列码记录：{}", subscriptionVO.getId());
                subscriptionVO.setAvailableDeviceAmount(0);
            }
            int total = collect.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())).mapToInt(o -> Objects.isNull(o.getTimes()) ? 0 : o.getTimes()).sum();
            int used = collect.stream().filter(c -> !c.getValidFlag().equals(LicenseCodeStatusEnum.Refunded.code())).mapToInt(o -> Objects.isNull(o.getUsedTimes()) ? 0 : o.getUsedTimes()).sum();
            subscriptionVO.setAvailableDeviceAmount(total - used);
        }
        return new PageInfo<>(subscriptionVOS);
    }

    @Override
    public PageInfo<SubscriptionResellerVO> pageForReseller(Integer page, Integer pageSize, String code) {
        VppMember member = vppMemberService.getCurrentUser();
        PageHelper.startPage(page, pageSize);
        List<SubscriptionResellerVO> list = vppSubscriptionMapper.listByCompanyForReseller(code, member.getCompanyId());
        List<String> subscriptionIds = list.stream().map(SubscriptionResellerVO::getId).collect(Collectors.toList());
        PageInfo<SubscriptionResellerVO> pageInfo = new PageInfo<>(list);
        List<LicenseCodes> licenseCodes = new ArrayList<>();
        if (!CollectionUtils.isEmpty(subscriptionIds)){
            licenseCodes = licenseCodeService.getBySubscriptionIds(subscriptionIds);
        }
        //根据订阅id查询相关序列码并且进行统计
        for (int i = 0; i < pageInfo.getList().size(); i++) {
            int total = 0 ;
            int active = 0 ;
            int refund = 0 ;
            SubscriptionResellerVO vo = pageInfo.getList().get(i);
            for (LicenseCodes licenseCode : licenseCodes) {
                LicenseCodeStatusEnum codeStatusEnum = LicenseCodeStatusEnum.getEnumByCode(licenseCode.getValidFlag()) ;
                 if(licenseCode.getSubscriptionId().equals(vo.getId())){
                     total = total + 1 ;
                     switch (codeStatusEnum) {
                         case ACTIVE:
                              active = active + 1;
                              break;
                         case Refunded:
                              refund  = refund + 1;
                              break;
                     }
                 }
            }
            pageInfo.getList().get(i).setActivatedNumber(active);
            pageInfo.getList().get(i).setTotalNumber(total);
            pageInfo.getList().get(i).setRefundNumber(refund);
        }
        return pageInfo;
    }

    @Override
    public Subscriptions getById(String id) {
        return vppSubscriptionMapper.selectByPrimaryKey(id);
    }


}
