package cn.kdan.pdf.tech.core.utils;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * @author Bob
 * @desc excel读取工具类
 */
public class ExcelUtils {

    /**
     * @author Bob
     * @desc 流读取Excel文件
     */
    public static List<List<String>> read(InputStream input, boolean isExcel2003) throws IOException {
        // 根据版本选择创建workbook
        Workbook wb = null;
        if (isExcel2003) {
            wb = new HSSFWorkbook(input);
        } else {
            wb = new XSSFWorkbook(input);
        }

        return readData(wb);
    }

    /**
     * @author Bob
     * @desc 读取数据
     */
    private static List<List<String>> readData(Workbook wb) {
        List<List<String>> dataList = new ArrayList<>();
        // 得到第一个shell
        Sheet sheet = wb.getSheetAt(0);
        // 得到Excel的行数
        // 总行数
        int totalRows = sheet.getPhysicalNumberOfRows();
        // 得到Excel的列数
        int totalCells;
        if (totalRows >= 1 && sheet.getRow(0) != null) {
            totalCells = sheet.getRow(0).getLastCellNum();
        } else {
            return null;
        }
        DecimalFormat df = new DecimalFormat("#");
        // 循环Excel的行
        for (int r = 0; r < totalRows; r++) {
            Row row = sheet.getRow(r);
            if (row == null) {
                continue;
            }
            // 行数据
            List<String> rowList = new ArrayList<>();

            // 循环Excel的列
            for (int c = 0; c < totalCells; c++) {
                Cell cell = row.getCell(c);
                String cellValue = "";

                // 判断数据类型，获取列数据
                if (null != cell) {
                    switch (cell.getCellType()) {
                        // 数字
                        case NUMERIC:
                            //修复可能会成为科学计数法的数字数据,
                            cellValue = df.format(cell.getNumericCellValue());
                            break;
                        // 字符串
                        case STRING:
                            cellValue = cell.getStringCellValue();
                            break;
                        // boolean
                        case BOOLEAN:
                            cellValue = cell.getBooleanCellValue() + "";
                            break;
                        // 公式
                        case FORMULA:
                            cellValue = cell.getCellFormula() + "";
                            break;
                        // 空值
                        case BLANK:
                            cellValue = "";
                            break;
                        // 错误
                        case ERROR:
                            cellValue = "非法字符";
                            break;

                        default:
                            cellValue = "未知类型";
                            break;
                    }
                }
                rowList.add(cellValue);
            }
            dataList.add(rowList);
        }
        return dataList;
    }

}
